package LATMOS::Accounts::Web::Controller::Users::Employment;
use Moose;
use namespace::autoclean;

use Date::Parse;
use DateTime;
use DateTime::TimeZone;
use POSIX;

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

LATMOS::Accounts::Web::Controller::Users::Employment - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub index :PathPart('employment') Chained('../users') Args(0) {
    my ( $self, $c ) = @_;

    $c->stash->{template} = 'users/employment.tt';
    my $base = $c->model('Accounts')->db;

    if (my $delemp = $c->req->param('delemployment')) { 
        $base->delete_object('employment', $delemp); 
        $base->commit; 
    }

    if (my $employment = 
        $c->stash->{user}->get_attributes('currentEmployment') ||
        $c->stash->{user}->get_attributes('nextEmployment')) {
        $c->detach('employment', [ $employment ]);
    } else {
        $c->forward('Emp');
    }
}

sub Emp :PathPart('employment') Chained('../users') CaptureArgs(0) {
    my ( $self, $c ) = @_;
    $c->stash->{template} = 'users/employment.tt';
    my $base = $c->model('Accounts')->db;
    $c->stash->{newemployment} = 1;

    $c->stash->{form} = $c->model('AttrForms', 'useremployment', $c->stash->{user});
    $c->stash->{forme} = $c->model('AttrForms', 'employment', 'employment', $base);
}

sub Create :PathPart('@create') Chained('Emp') Args(0) {
    my ($self, $c) = @_;

    my $username = $c->stash->{username};
    
    for (1) {
        $c->req->params->{firstday} =~ s:^(\d+)/(\d+)/(\d+):$3-$2-$1:;
        $c->req->params->{lastday}  =~ s:^(\d+)/(\d+)/(\d+):$3-$2-$1:;
        my $nFirstday = $c->req->param('firstday');
        # Date::Parse understand only mm/dd/yyyy...
        my $time = str2time($nFirstday) or last;
        my $lastd = DateTime->from_epoch( epoch => $time )
            ->set_time_zone( DateTime::TimeZone->new( name => 'local' ) );
        $lastd->subtract(days => 1);

        my ($empl) = $c->stash->{user}->listEmployment;
        if ($empl) {
            my $obj = $c->model('Accounts')->db->get_object('employment', $empl);
            # We try to truncate last employment if need
            # if new employment if after only

            if ($obj->get_attributes('firstday') lt $lastd->ymd('-')) {
                if (my $lastday = $obj->get_attributes('lastday')) {
                    my $firstday = $obj->get_attributes('firstday');
                    if ($lastd->ymd('-') ge $firstday &&
                        $lastday         ge $lastd->ymd('-')) {
                        $obj->set_c_fields(lastday => $lastd->ymd('-'));
                    }
                } elsif (!$obj->get_attributes('lastday')) {
                    $obj->set_c_fields(lastday => $lastd->ymd('-'));
                }
            }
        }

        my $id = $username . '-' . $lastd->ymd('-');
        $c->req->params->{user} = $username;
        if ($id = $c->stash->{forme}->set_attrs([ 'user',
                    map { $_->name } $c->stash->{forme}->write_attributes], $id)) {
            $c->res->redirect($c->uri_for('/users', $username, 'employment', $id));

            $c->stash->{form}->set_attrs;
            
            return 1;
        }
    }
}

sub New :PathPart('@new') Chained('Emp') Args(0) {
    my ($self, $c) = @_;
}

sub Duplicate :PathPart('@duplicate') Chained('Emp') Args(0) {
    my ($self, $c) = @_;

    my $oldempl = $c->model('Accounts')->db->get_object('employment',
        $c->req->param('source')
        ? $c->req->param('source')
        : ($c->stash->{user}->listEmployment)[0])
        or return;
    foreach (qw(company managerContact contratType department)) {
        $c->req->params->{$_} = $oldempl->get_attributes($_);
    }
    if (my $lastday = $oldempl->get_attributes('lastday')) {
        my $time = DateTime->from_epoch(epoch => str2time($lastday));
        $time->set_time_zone( DateTime::TimeZone->new( name => 'local' ) );
        $time->add( days => 1 );
        $c->req->params->{firstday} = $time->ymd;
    }
    $c->stash->{forme}->set_attrs;
}

sub employment :PathPart() Chained('../users') Args(1) {
    my ( $self, $c, $employment ) = @_;

    $c->stash->{template} = 'users/employment.tt';
    my $base = $c->model('Accounts')->db;

    $c->stash->{employment} = $base->get_object('employment', $employment);

    $c->stash->{form} = $c->model('AttrForms', 'useremployment', $c->stash->{user});
    $c->stash->{form}->set_attrs;

    $c->stash->{forme} = $c->model('AttrForms', 'employment', $c->stash->{employment});
    $c->stash->{forme}->set_attrs;
}



=encoding utf8

=head1 AUTHOR

Olivier Thauvin,Guyancourt - B1428,+33 1 80285052,

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
